<?php
/**
 * @version     6.0 +
 * @package       Open Source Excellence Security Suite
 * @subpackage    Open Source Excellence CPU
 * @author        Open Source Excellence {@link http://www.opensource-excellence.com}
 * @author        Created on 30-Sep-2010
 * @author        Updated on 30-Mar-2013 
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 * @copyright Copyright (C) 2008 - 2010- ... Open Source Excellence
 *
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
if (!defined('_JEXEC') && !defined('OSE_ADMINPATH')) {
	die('Direct Access Not Allowed');
}
class oseFirewall {
	private $ip = null;
	private $ip32 = null;
	private $ipStatus = null;
	private $url = null;
	private $referer = null;
	private $tags = null;
	private $target = null;
	private $converters = array ();
	private $allowExts = null;
	private $logtime = null;
	protected $db = null;
	private $json = null;
	protected $threshold = 35;
	protected $blockIP = true;
	private $aclid = null;
	protected $scanGoogleBots = true;
	protected $scanYahooBots = true;
	protected $scanMsnBots = true;
	protected $devMode = false;
	protected $slient_max_att = 10;
	protected $banpagetype = false;
	protected $sfspam = false;
	protected $sfs_confidence = 30;
	protected $visits = 0;
	protected $blockMode = 1;
	protected $replaced  = array(); 
	public function __construct() {
		$this->initSetting();
	}
	private function initSetting() {
		$this->setTargetURL();
		$this->setReferer();
		$this->setClientIP();
		$this->setDBO();
		$this->setConfig();
	}
	private function setConfig() {
		$query = 'SELECT `key`, `value` FROM `#__ose_secConfig` WHERE `type` IN ("seo", "scan", "addons")';
		$this->db->setQuery($query);
		$results = $this->db->loadAssocList();
		foreach ($results as $result)
		{
			$key = $result['key']; 
			if (in_array($key, array('threshold', 'slient_max_att', 'sfs_confidence')))
			{
				$this->$key = (int) $result['value'] ;
			}
			else
			{
				$this->$key = $result['value'] ;
			} 
		}
	}
	private function setTargetURL() {
		$this->url = ((!empty($_SERVER['HTTPS'])) ? "https://" : "http://") . str_replace('?' . $_SERVER['QUERY_STRING'], '', $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
	}
	private function setReferer() {
		if (isset ($_SERVER['HTTP_REFERER'])) {
			$this->referer = $_SERVER['HTTP_REFERER'];
		} else {
			$this->referer = 'N/A';
		}
	}
	private function setClientIP() {
		$ipmanager = new oseFirewallIpManager();
		$this->ip = $ipmanager->getIP();
		$this->ip32 = $ipmanager->getIPLong(true);
		$this->ipStatus = $ipmanager->getIPStatus();
		$this->aclid = $ipmanager->getACLID();
	}
	private function setDBO() {
		$this->db = oseDBO :: instance();
	}
	public function hackScan() {
		$continue = $this->checkContinue();
		if ($continue === false) {
			if (class_exists('oseDBO')) {
				//$this->db->__destruct();
			}
			return;
		}
		else if ($this->ipStatus == 1) {
			$this->showBanPage();
		}
		else
		{
			$this->scanAttack();
		}
		if (class_exists('oseDBO')) {
			$this->db->__destruct();
		}
	}
	private function scanAttack()
	{
			$scanner = new oseFirewallScanner($this->db, $this->ip);
			if ($this->sfspam) {
				$scanResult = $scanner->isSpambots();
				if ($scanResult != false) {
					$this->addACLRule(1, $scanResult['impact']);
					// Spamming ID is 11 in attacktype table; 
					$this->addL1DetContent(11, $scanResult['detcontent_content'], $scanResult['rule_id']);
					$scanner->resetReturn();
					$this -> controlAttack();
				}
			}
			unset ($scanResult);
			$scanResult = $scanner->ScanLayer1();
			if (!empty ($scanResult)) {
				$this->addACLRule($this->blockIP, $scanResult['impact']);
				$this->addL1DetContent(1, $scanResult['detcontent_content'], $scanResult['rule_id']);
				$scanner->resetReturn();
				$this -> controlAttack();
			}
			unset ($scanResult);
			$scanResult = $scanner->ScanLayer2();
			if (!empty ($scanResult['result'])) {
				$this->addACLRule($this->blockIP, $scanResult['impact']);
				foreach ($scanResult['result'] as $result) {
					$detcontent_id = $this->addL2DetContent($result['attacktype'], $result['detcontent_content'], $result['rule_id'], $result['var']);
					$scanner->resetReturn();
				}
				$this -> controlAttack();
			}
			unset ($scanResult);
			if (!empty($_FILES))
			{
				// Scan if file types are allowed
				$scanResult = $scanner->checkFileTypes($_FILES);
				if ($scanResult!=false) {
				  	$this->addACLRule(1, $scanResult['impact']);
				  	$this->addL1DetContent(13, $scanResult['detcontent_content'], $scanResult['rule_id']);
				  	$scanner->resetReturn();
				  	$this -> controlAttack();
				}
				unset ($scanResult);
				// Scan file againsts virus database
				$scanResult = $scanner->scanFileVirus($_FILES);
				if ($scanResult!=false) {
				  	$this->addACLRule(1, $scanResult['impact']);
					$this->addL1DetContent(14, $scanResult['detcontent_content'], $scanResult['rule_id']);
					$scanner->resetReturn();
					$this -> controlAttack();
				}
				unset ($scanResult);
			}
			if (isset($this->antiflooding) && $this->antiflooding==true)
			{
				$scanResult = $scanner -> scanFlooding();
				if ($scanResult!=false) {
				  	$this->addACLRule(1, $scanResult['impact']);
					$this->addL1DetContent(9, $scanResult['detcontent_content'], $scanResult['rule_id']);
					$scanner->resetReturn();
					$this -> controlAttack();
				}
				unset ($scanResult);
			}
	}
	private function controlAttack() {
		$score = $this->getScore();
		$visits = $this->getVisits();
		$notified = $this->getNotified();
		if ($score < $this->threshold)
		{
			return; 
		}
		// Ensure everything is cleaned before moving on; 
		switch ($this->blockIP)
		{
			case 1:
				$this -> filterAttack(false);
				$this -> sendEmail('blacklisted', $notified); 
				$this -> showBanPage(); 
			break;
			case 0:
				$this -> filterAttack(false);
				$this -> sendEmail('403blocked', $notified); 
				$this -> show403Page();
			break;
			case 2:
				if ($visits <= $this->slient_max_att)
				{
					$this -> updateVisits();
					$url = $this -> filterAttack(true);
					$this -> sendEmail('filtered', $notified);
					$this->redirect($url);
				}
				else
				{
					$this -> filterAttack(false);
					$this -> updateStatus(1); 
					$this -> sendEmail('blacklisted', $notified);
					$this -> showBanPage();
				}
			break;
		}
	}
	private function filterAttack($redirect)
	{
		$db = oseDBO :: instance();
		$query= "SELECT `attacktypeid`,`rule_id`, `keyname`, `content` FROM `#__osefirewall_attackmap` WHERE `aclid` = ". $this->aclid.' GROUP BY `rule_id` ORDER BY LENGTH(`content`) DESC';
		$db->setQuery($query);
		$results = $db->loadObjectList();
		foreach ($results as $result)
		{
			if($result->attacktypeid==1)
			{
				$url = $this->convertL1Attack($result->content);
				if ($redirect==true)
				{
					return $url; 
				}
			}
			elseif (!empty($result->keyname) && !empty($result->rule_id))
			{
				$this->convertL2Attack($result->rule_id, $result->keyname);
			}
		} 
	}
	private function convertL1Attack($content)
	{
		$content = $this->cleanVariable($content);
		$this->replaced['original']['URL'] = ((!empty($_SERVER['HTTPS'])) ? "https://" : "http://") . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		$_SERVER['REQUEST_URI'] = $this->cleanVariable($_SERVER['REQUEST_URI']);
		$_SERVER['QUERY_STRING'] = $this->cleanVariable($_SERVER['QUERY_STRING']); 
		$_SERVER['REQUEST_URI'] = str_replace($content, '', $_SERVER['REQUEST_URI']);
		$_SERVER['QUERY_STRING'] = str_replace($content, '', $_SERVER['QUERY_STRING']);
		$redirect = ((!empty($_SERVER['HTTPS'])) ? "https://" : "http://") . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		$this->replaced['filtered']['URL'] = $redirect;
		return $redirect; 
	}
	private function convertL2Attack($rule_id, $keyname)
	{
		$tmp = explode('.', $keyname);
		if (!empty($tmp) && is_array($tmp))
		{ 
			switch($tmp[0])
			{
				case 'GET':
				   $this->replaced['original']['GET'][$tmp[1]] = $_GET[$tmp[1]];
				   $_GET[$tmp[1]] = $this->filterVariable($_GET[$tmp[1]], $rule_id);
				   $this->replaced['filtered']['GET'][$tmp[1]] = $_GET[$tmp[1]];
				break; 
				case 'POST':
				   $this->replaced['original']['POST'][$tmp[1]] = $_POST[$tmp[1]];	
				   $_POST[$tmp[1]] = $this->filterVariable($_POST[$tmp[1]], $rule_id);
				   $this->replaced['filtered']['POST'][$tmp[1]] = $_POST[$tmp[1]]; 
				break; 
			}
		}
	}
	private function filterVariable($var, $rule_id)
	{
		$pattern = '/'.$this -> getPattern($rule_id).'/ims'; 
		$var = preg_replace($pattern, '', $var);
		return $var; 
	}
	private function getPattern($rule_id)
	{
		$query = "SELECT `filter` FROM `#__osefirewall_filters` WHERE `id` = " . (int) $rule_id;
		$this->db->setQuery($query);
		$filter =  $this->db->loadResult();
		return $filter;   
	}
	private function cleanVariable($var)
	{
		return html_entity_decode(urldecode($var)); 
	}
	private function redirect($url)
	{
		$url = str_replace("null", "", $url);
		header('Location: '.$url);
	}
	private function updateStatus($status)
	{
		$varValues = array (
				'status' => (int)$status
			);
		$result = $this->db->addData('update', '#__osefirewall_acl', 'id', $this->aclid, $varValues);
		return (boolean)$result;
	}
	private function getScore()
	{
		$db = oseDBO :: instance();
		$query = "SELECT `score` FROM `#__osefirewall_aclipmap` WHERE `id` = ". (int)$this->aclid; 
		$db->setQuery($query);
		$score = $db->loadResult();
		return (int)$score; 
	}
	private function getVisits()
	{
		$db = oseDBO :: instance();
		$query = "SELECT `visits` FROM `#__osefirewall_aclipmap` WHERE `id` = ". (int)$this->aclid; 
		$db->setQuery($query);
		$visits = $db->loadResult();
		return (int)$visits; 
	}
	private function getNotified()
	{
		$db = oseDBO :: instance();
		$query = "SELECT `notified` FROM `#__osefirewall_aclipmap` WHERE `id` = ". (int)$this->aclid; 
		$db->setQuery($query);
		$notified = $db->loadResult();
		return (int)$notified;
	}
	private function updateVisits()
	{
		$db = oseDBO :: instance();
		$query = "UPDATE `#__osefirewall_aclipmap` SET `visits` = (`visits` +1) WHERE `id` = ". (int)$this->aclid; 
		$db->setQuery($query);
		$result = $db->query(); 
		return (boolean)$result; 
	}
	private function addACLRule($status, $score) {
		$page_id = $this->addPages();
		$referer_id = $this->addReferer();
		if (empty ($this->aclid)) {
			$varValues = array (
				'id' => $this->aclid,
				'name' => $this->ip,
				'datetime' => date('Y-m-d h:i:s'),
				'score' => (int)$score,
				'status' => (int) $status,
				'referers_id' => $referer_id,
				'pages_id' => $page_id,
				'visits' => 1 
			);
			$ipmanager = new oseFirewallIpManager();
			$aclid = $ipmanager->getACLID();
			if (empty($aclid))
			{
				$this->aclid = $this->db->addData('insert', '#__osefirewall_acl', null, null, $varValues);
				if (!empty ($this->aclid)) {
					$ipmanager->addIP('ip', $this->aclid);
				}
			}
		}
		else
		{
			$varValues = array (
				'score' => (int)$score,
				'status' => (int) $status
			);
			$this->aclid = $this->db->addData('update', '#__osefirewall_acl', 'id', $this->aclid, $varValues);
		}
	}
	private function updateACLScore($score)
	{
		$varValues = array (
				'score' => $score
			);
		$this->db->addData('update', '#__osefirewall_acl', 'id', $this->aclid, $varValues);
	}
	private function addL1DetContent($attacktypeID, $detcontent_content = null, $rule_id = null) {
		$exists = $this->isDetContentExists($attacktypeID, $rule_id);
		if (!empty ($exists)) {
			return;
		}
		$detattacktype_id = $this->insertDetAttacktype($attacktypeID);
		if (!empty ($detattacktype_id)) {
			$this->insertDetected($detattacktype_id);
			if (!empty ($detcontent_content) && !empty ($rule_id)) {
				$this->insertDetContentDetail($detattacktype_id, $detcontent_content, $rule_id, null);
			}
		}
		return $detattacktype_id;
	}
	private function addL2DetContent($attacktypeIDArray, $detcontent_content, $rule_id, $varKey) {
		$attacktypeIDArray = oseJSON :: decode($attacktypeIDArray);
		foreach ($attacktypeIDArray as $attacktypeID) {
			$exists = $this->isDetContentExists($attacktypeID, $rule_id);
			if ($exists==true) {
				continue;
			} else {
				$detattacktype_id = $this->insertDetAttacktype($attacktypeID);
				if (!empty ($detattacktype_id)) {
					$this->insertDetected($detattacktype_id);
					if (!empty ($detcontent_content) && !empty ($rule_id)) {
						$var_id = $this->insertVarKey($varKey);
						$this->insertDetContentDetail($detattacktype_id, $detcontent_content, $rule_id, $var_id);
					}
				}
			}
		}
	}
	private function insertVarKey($varKey) {
		$query = 'SELECT `id` FROM `#__osefirewall_vars` WHERE `keyname` = ' . $this->db->Quote($varKey);
		$this->db->setQuery($query);
		$id = $this->db->loadResult();
		if (empty ($id)) {
			$varValues = array (
				'id' => 'DEFAULT',
				'keyname' => $varKey,
				'status' => 1
			);
			$id = $this->db->addData('insert', '#__osefirewall_vars', null, null, $varValues);
			return $id;
		} else {
			return $id;
		}
	}
	private function insertDetAttacktype($attacktypeID) {
		$varValues = array (
			'id' => 'DEFAULT',
			'attacktypeid' => (int) $attacktypeID
		);
		$detattacktype_id = $this->db->addData('insert', '#__osefirewall_detattacktype', null, null, $varValues);
		return $detattacktype_id;
	}
	private function insertDetected($detcontent_id) {
		$varValues = array (
			'acl_id' => (int) $this->aclid,
			'detattacktype_id' => (int) $detcontent_id
		);
		$this->db->addData('insert', '#__osefirewall_detected', null, null, $varValues);
	}
	private function isDetContentExists($attacktypeID, $rule_id = null) {
		$where = array ();
		$where[] = '`aclid` = ' . (int) $this->aclid;
		$where[] = '`attacktypeid` = ' . (int) $attacktypeID;
		if (!empty ($rule_id)) {
			$where[] = '`rule_id` = ' . (int) $rule_id;
		}
		$where = $this->db->implodeWhere($where);
		$query = 'SELECT COUNT(`aclid`) FROM `#__osefirewall_attackmap` ' . $where;
		$this->db->setQuery($query);
		$result = $this->db->loadResult();
		return $result;
	}
	private function insertDetContentDetail($detattacktype_id, $detcontent, $rule_id, $var_id = null) {
		$detcontent_id = $this->insertDetContent($detcontent);
		$varValues = array (
			'detattacktype_id' => (int) $detattacktype_id,
			'detcontent_id' => $detcontent_id,
			'rule_id' => $rule_id
		);
		if (!empty ($var_id)) {
			$varValues['var_id'] = $var_id;
		}
		$this->db->addData('insert', '#__osefirewall_detcontdetail', null, null, $varValues);
		return;
	}
	private function insertDetContent($detcontent) {
		$query = 'SELECT `id` FROM `#__osefirewall_detcontent` WHERE `content` = ' . $this->db->Quote($detcontent, true);
		$this->db->setQuery($query);
		$id = $this->db->loadResult();
		if (empty ($id)) {
			$varValues = array (
				'id' => 'DEFAULT',
				'content' => $detcontent
			);
			$id = $this->db->addData('insert', '#__osefirewall_detcontent', null, null, $varValues);
			return $id;
		} else {
			return $id;
		}
	}
	private function addPages() {
		$query = 'SELECT `id`, `visits` FROM `#__osefirewall_pages` WHERE `page_url` = ' . $this->db->Quote($this->url, true);
		$this->db->setQuery($query);
		$results = $this->db->loadObject();
		if (empty ($results)) {
			$varValues = array (
				'id' => 'DEFAULT',
				'page_url' => $this->url,
				'action' => 1,
				'visits' => 1
			);
			$id = $this->db->addData('insert', '#__osefirewall_pages', null, null, $varValues);
		} else {
			$varValues = array (
				'visits' => $results->visits + 1
			);
			$id = $this->db->addData('update', '#__osefirewall_pages', 'id', $results->id, $varValues);
		}
		return $id;
	}
	private function addReferer() {
		$query = 'SELECT `id` FROM `#__osefirewall_referers` WHERE `referer_url` = ' . $this->db->Quote($this->referer, true);
		$this->db->setQuery($query);
		$results = $this->db->loadObject();
		if (empty ($results)) {
			$varValues = array (
				'id' => 'DEFAULT',
				'referer_url' => $this->referer
			);
			$id = $this->db->addData('insert', '#__osefirewall_referers', null, null, $varValues);
		} else {
			$id = $results->id;
		}
		return $id;
	}
	private function getAllowBots() {
		$bots = array ();
		if ($this->scanGoogleBots === false) {
			$bots[] = 'Google';
		}
		if ($this->scanMsnBots === false) {
			$bots[] = 'msnbot';
		}
		if ($this->scanYahooBots === false) {
			$bots[] = 'Yahoo';
		}
		return $bots;
	}
	private function set($var, $value) {
		$this-> $var = $value;
	}
	private function get($var) {
		return $this-> $var;
	}
	private function checkContinue() {
		if ($this->devMode == true) {
			return false;
		}
		$bots = $this->getAllowBots();
		$ipmanager = new oseFirewallIpManager();
		if (COUNT($bots) > 0 && $ipmanager->isSearchEngineBot(implode('|', $bots), $_SERVER['HTTP_USER_AGENT'])) {
			$this->set('blockIP', 2);
			$this->set('slient_max_att', 20);
			return false;
		}
		if ($this->ipStatus == 3) {
			return false;
		}
		if((preg_match("/administrator\/*index.?\.php$/", $_SERVER['SCRIPT_NAME'])) || !empty($_SESSION['secretword']))
		{
			if(isset($this->secretword) && $this->secretword == $_SERVER['QUERY_STRING'])
			{
				$_SESSION['secretword'] = $this->secretword;
				return false; 
			}
		}
		if (isset ($_REQUEST['option'])) {
			$components = $this->getWLComponents () ;
			$components = array_merge ($components, array (
					'com_ose_fileman',
					'com_ose_antihacker',
					'com_ose_antivirus',
					'com_civicrm'
			));
			if (COUNT($_REQUEST['option']) == 1 && (in_array($_REQUEST['option'], $components))) {
				return false;
			} else {
				return true;
			}
		} else {
			return true;
		}
	}
	private function getWLComponents () {
		$query = 'SELECT `comname` FROM `#__osefirewall_components` ';
		$this->db->setQuery($query);
		$results = $this->db->loadObjectList();
		$return = array(); 
		if (!empty($results))
		{
			foreach ($results as $result)
			{
				$return[] = $result->comname;
			} 
		}
		return $return; 
	}
	private function showBanPage() {
		$adminEmail = (isset ($this->adminEmail)) ? $this->adminEmail: '';
		$customBanPage = (!empty ($this->customBanpage)) ? $this->customBanpage: 'Banned';
		$pageTitle = (!empty ($this->pageTitle)) ? $this->pageTitle : 'OSE Security Suite';
		$metaKeys = (!empty ($this->metaKeywords)) ? $this->metaKeywords : 'OSE Security Suite';
		$metaDescription = (!empty ($this->metaDescription)) ? $this->metaDescription : 'OSE Security Suite';
		$metaGenerator = (!empty ($this->metaGenerator)) ? $this->metaGenerator : 'OSE Security Suite';
		$banhtml = $this->getBanPage($adminEmail, $pageTitle, $metaKeys, $metaDescription, $metaGenerator, $customBanPage); 
		echo $banhtml; exit;
	}
	private function getBanPage($adminEmail, $pageTitle, $metaKeys, $metaDescription, $metaGenerator, $customBanPage)
	{
		$banbody = $this->getBanPageBody($customBanPage, $adminEmail);
		$banhtml = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
					<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
						<head>
							  <meta http-equiv="content-type" content="text/html; charset=utf-8" />
							  <meta name="robots" content="index, follow" />
							  <meta name="keywords" content="' . $metaKeys . '" />
							  <meta name="description" content="' . $metaDescription . '" />
							  <meta name="generator" content="' . $metaGenerator . '" />
							  <title>' . $pageTitle . '</title>
						</head>
						<body>
						'.$banbody.'	  
						</body>
					</html>';
		return $banhtml;
	}
	private function getBanPageBody($customBanPage, $adminEmail)
	{
		$banbody = '<div style="margin:auto;width:780px;border:0px solid #0082b0;padding:0px 10px 10px 10px;z-index:100;color:#000000;">
							  <br/>
								' . $customBanPage . '
							  <div style="font-family: arial,helvetica,sans-serif;background-color:#ffffff;padding: 10px 0px 0px 0px" align="center"><font color="#666666" size="1">Your IP address is ' . $this->ip . '. If you believe this is an error, please contact the <a href="mailto:' . $adminEmail . '?Subject=Inquiry:%20Banned%20for%20suspicious%20hacking%20behaviour - IP: ' . $this->ip . ' - Violation"> Webmaster </a>.</font></div>
					</div>';
		return $banbody;
	}
	private function show403Page() {
		$adminEmail = (isset ($this->adminEmail)) ? $this->adminEmail: '';
		$customBanPage = (!empty ($this->customBanpage)) ? $this->customBanpage: 'Banned';
		$banbody = $this->getBanPageBody($customBanPage, $adminEmail);
		header('HTTP/1.1 403 Forbidden');
		$banbody  = "<html>
						<head>
							<title>403 Forbidden</title>
						</head>
						<body>
								".$banbody."
						</body>
					 </html>";
		echo $banbody;			 
		exit;
	}
	private function sendEmail($type, $notified)
	{
		$config_var = $this->getConfigVars(); 
		$oseCPU = new oseCPU(); 
		$oseCPU -> callLibClass('emails', 'oseEmail');
		$oseEmail = new oseEmail('antihack');
		$email = $oseEmail->getEmailByType($type);
		$email = $this->convertEmail($email, $config_var);
		$result = $oseEmail->sendMail($email, $config_var);
		if ($result == true)
		{
			$this->updateNotified(1); 
		}
	}
	private function updateNotified($status)
	{
		$varValues = array (
				'notified' => (int)$status
			);
		$result = $this->db->addData('update', '#__osefirewall_acl', 'id', $this->aclid, $varValues);
		return (boolean)$result;
	}
	private function convertEmail($email, $config_var)
	{
		$attackType = $this->getAttackTypes();
		$attackType = implode(',', $attackType); 
		$ipURL = $this ->getIPURL($config_var); 
		$violation = $this->getViolation();
		$totalImpact = $this->getScore(); 
		$email->subject = $email->emailSubject." for [".$_SERVER['HTTP_HOST']."]";
		$email->body = str_replace('[attackType]', $attackType, $email->emailBody);
		$email->body = str_replace('[violation]', $violation, $email->body);
		$email->body = str_replace('[logtime]', $this->logtime, $email->body);
		$email->body = str_replace('[ip]', $this->ip, $email->body);
		$email->body = str_replace('[target]', $this->target, $email->body);
		$email->body = str_replace(array('[referrer]', '[referer]'), $this->referer, $email->body);
		$email->body = str_replace('[aclid]', $ipURL, $email->body);
		$email->body = str_replace('[score]', $totalImpact, $email->body);
		return $email; 
	}
	private function getViolation()
	{
		$return = '';
		if (!empty($this->replaced['original']))
		{
			foreach ($this->replaced['original'] as $key => $replaced)
			{
				if ($key =='URL')
				{
					$return .= $key. ' <br />redirected from <font color="red">'.$this->replaced['original'][$key]. '</font> <br />to '. $this->replaced['filtered'][$key].'<br/>'; 
				}
				elseif (in_array($key, array('GET', 'POST')))
				{
					foreach ($replaced as $k => $v)
					{
						$return .= $key.'.'.$k.' changed from <font color="red">'.$this->replaced['original'][$key][$k].'</font> to '. $this->replaced['filtered'][$key][$k].'<br/>';
					}
				}
 		}
		}
 		return $return; 
	}
	private function getIPURL($config_var)
	{
		return "<a href='".$config_var->live_site."/administrator/index.php?option=com_ose_antihacker&view=manageips&id=".$this->aclid."'>".$this->aclid."</a>";
	}
	private function getAttackTypes()
	{
		$query = 'SELECT DISTINCT `name` FROM `#__osefirewall_attackmap` WHERE `aclid` = '.(int)$this->aclid; 
		$this->db->setQuery($query); 
		$results = $this->db -> loadResultArray(); 
		return $results;  	
	}
	private function getConfigVars()
	{
		if (class_exists('SConfig'))
		{
			$config = new SConfig(); 
			return $config; 
		}
		elseif (class_exists('JConfig'))
		{
			$config = new JConfig(); 
			return $config; 
		}
	}
}