<?php
/**
 * @version     6.0 +
 * @package       Open Source Excellence Security Suite
 * @subpackage    Open Source Excellence CPU
 * @author        Open Source Excellence {@link http://www.opensource-excellence.com}
 * @author        Created on 30-Sep-2010
 * @author        Updated on 30-Mar-2013 
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 * @copyright Copyright (C) 2008 - 2010- ... Open Source Excellence
 *
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
if (!defined('_JEXEC') && !defined('OSE_ADMINPATH')) {
	die("Direct Access Not Allowed");
}
class oseFirewallScanner {
	private $db = null;
	private $ip = null;
	private $ScanReturn = array ();
	public function __construct($db, $ip) {
		$this->db = $db;
		$this->ip = $ip;
		$configs = $this->getConfiguration();
		if (!empty ($configs)) {
			foreach ($configs as $config) {
				if (strstr($config['key'], 'convert')) {
					if ($config['value'] == true || $config['value'] == '1') {
						$this->converters[] = $config['key'];
					}
				}
				elseif (strstr($config['key'], 'slient_max_att')) {
					$this->slient_max_att = (empty ($config['value'])) ? 10 : $config['value'];
				} else {
					$this-> $config['key'] = $config['value'];
				}
			}
		}
		unset ($configs);
	}
	private function getConfiguration() {
		$this->db->setQuery('SELECT * FROM `#__ose_secConfig` ');
		$results = $this->db->loadAssocList();
		return $results;
	}
	public function resetReturn()
	{
		$this->ScanReturn = array();
	}
	// Functions to check spambots //
	public function isSpambots() {
		return $this->CheckIsSpambot();
	}
	private function CheckIsSpambot() {
		// Initiate and declare spambot/errorDetected as false - as we're just getting started
		$isspamcheck = $this ->isspamcheck();
		$spambot = false;
		if ($spambot != true && $this->ip != "" && $isspamcheck == false) {
			$data = array ();
			$data["ip"] = $this->ip;
			$data["f"] = 'json';
			$json_return = $this->posttoSFS($data);
			$result = oseJSON :: decode($json_return);
			if (!isset($result->ip->confidence)) {
				$spambot = false;
			}
			elseif ($result->ip->appears == 1 && $result->ip->confidence >= (int)$this->sfs_confidence) // Was the result was registered
			{
				$spambot = true; // Check failed. Result indicates dangerous.
				$this->ScanReturn['detcontent_content'] = oseJSON::encode($result->ip); 
				$this->ScanReturn['rule_id'] = 1;
				$this->ScanReturn['impact'] = 100;
				$this->updatespamcheck(true);
				return $this->ScanReturn; 
			} else {
				$this->updatespamcheck(true);
				$spambot = false; // Check passed. Result returned safe.
			}
			unset ($data);
			unset ($json_return);
			unset ($result);
		}
		return $spambot; // Return test results as either true/false or 1/0
	}
	private function isspamcheck()
	{
		$this->ip32 = ip2long( $this -> ip);
		$query = "SELECT `ischecked` FROM `#__osefirewall_sfschecked`` WHERE `ip` = " . $this->db->Quote($this->ip32, true);
		$this->db->setQuery($query);
		$result = $this->db->loadResult();
		return ($result == 1) ? true : false;
	}
	private function updatespamcheck($ischeck)
	{
		$varValues = array (
				'ip32_start' => $this->ip32,
				'ischecked' => 1
			);
		$this->db->addData('insert', '#__osefirewall_sfschecked', '', '', $varValues);
	}
	private function posttoSFS($data) {
		$Url = "http://www.stopforumspam.com/api?" . http_build_query($data);
		$Curl = curl_init();
		curl_setopt($Curl, CURLOPT_URL, $Url);
		curl_setopt($Curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($Curl, CURLOPT_TIMEOUT, 4);
		curl_setopt($Curl, CURLOPT_FAILONERROR, 1);
		$ResultString = curl_exec($Curl);
		curl_close($Curl);
		unset ($Url);
		unset ($Curl);
		return $ResultString;
	}
	// Functions to scan layer 2 attacks //
	public function ScanLayer2() {
		if ($this->scancookies == true) {
			$request = array (
				'GET' => $_GET,
				'POST' => $_POST,
				'COOKIE' => $_COOKIE
			);
		} else {
			$request = array (
				'GET' => $_GET,
				'POST' => $_POST
			);
		}
		$l2Patterns = $this->getPatterns(1, true);
		$this->ScanReturn['impact'] = 0;
		$this->ScanReturn['result'] = array();  
		foreach ($request as $key => $value) {
			$this->ScanLayer2Iterate($l2Patterns, $key, $value, null, null);
		}
		unset ($l2Patterns);
		return $this->ScanReturn;
	}
	private function ScanLayer2Iterate($l2Patterns, $key, $value, $priKey = null, $secKey = null) {
		if (!is_array($value)) {
			if (is_string($value)) {
				$this->layer2DetectAttack($l2Patterns, $key, $value, $priKey, $secKey);
			} else {
				return false;
			}
		} else {
			foreach ($value as $subKey => $subValue) {
				$this->ScanLayer2Iterate($l2Patterns, $key . '.' . $subKey, $subValue, $key, $subKey);
			}
		}
	}
	private function layer2DetectAttack($l2Patterns, $key, $value, $priKey = null, $secKey = null) {
		if (empty ($value)) {
			return;
		}
		// check if this field is part of the exceptions
		$tmptest = explode('.', $key);
		if (count($tmptest) > 1) {
			$key = $tmptest[0] . '.' . $tmptest[1];
		}
		unset ($tmptest);
		$isWhitelisted = $this->checkIsWhitelistedVars($key);
		if ($isWhitelisted == true) {
			return;
		}
		// check for magic quotes and remove them if necessary
		if (function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc()) {
			$value = stripslashes($value);
		}
		if (function_exists('get_magic_quotes_gpc') && !get_magic_quotes_gpc() && version_compare(PHP_VERSION, '5.3.0', '>=')) {
			$value = preg_replace('/\\\(["\'\/])/im', '$1', $value);
		}
		if (!empty ($this->converters)) {
			$value = IDS_Converter :: runAll($value);
			$value = $this->runCentrifuge ? IDS_Converter :: runCentrifuge($value) : $value;
		}
		// scan keys if activated via config
		if (!isset ($this->scankey)) {
			$this->scankey = false;
		}
		if (!isset ($this->scankeyLength)) {
			$this->scankeyLength = 100;
		}
		if ($this->scankey == true) {
			if (!empty ($this->converters)) {
				$key = $this->scankey ? IDS_Converter :: runAll($key) : $key;
				$key = ($this->scankey && $this->runCentrifuge) ? IDS_Converter :: runCentrifuge($key) : $key;
			}
			if (strlen($key) > $this->scankeyLength) {
				$this->show403('Illegal Key detected.');
			}
		}
		// scan value against Patterns;
		$i = 0;
		if (!empty ($l2Patterns)) {
			foreach ($l2Patterns as $pattern) {
				$this->checkIsPatternMatch($key, $value, $pattern);
			}
		}
		unset ($key);
		unset ($value);
		unset ($l2Patterns);
	}
	private function getPatterns($action, $obj) {
		$query = "SELECT `id`,`filter`,`attacktype`,`impact` FROM `#__osefirewall_filters` WHERE `action` = " . (int) $action;
		$this->db->setQuery($query);
		if ($obj == true) {
			$results = $this->db->loadObjectList();
		} else {
			$results = $this->db->loadResultArray();
		}
		unset ($query);
		return $results;
	}
	private function checkIsPatternMatch($key, $value, $pattern) {
		$match = false;
		if ($this->scankey == true) {
			$testPatterns = $this->match($key, $pattern->filter);
			if (!empty ($testPatterns)) {
			}
		}
		$testPatterns = $this->match($value, $pattern->filter);
		$num = count($this->ScanReturn['result']);
		if (!empty ($testPatterns)) {
			$this->ScanReturn['result'][$num]['detcontent_content'] = implode("|", $testPatterns);
			$this->ScanReturn['result'][$num]['rule_id'] = $pattern->id;
			$this->ScanReturn['result'][$num]['attacktype'] = $pattern->attacktype;
			$this->ScanReturn['result'][$num]['var'] = $key;
			$this->ScanReturn['impact'] += $pattern->impact;
			
		}
		return;
	}
	private function match($string, $filter) {
		if (!is_string($string)) {
			$this->show403('Invalid argument provided');
		}
		preg_match('/' . $filter . '/ims', strtolower($string), $matches);
		return $matches;
	}
	// Whitelisted varialbes are defined as 3 in this table; 
	private function checkIsWhitelistedVars($var) {
		if (preg_match("/[a-zA-Z]+\d+/", $var, $matches)>0)
		{
			$var = preg_replace("/\d+/", "{num}", $var);
			$query = "SELECT * FROM `#__osefirewall_vars` WHERE `keyname` = " . $this->db->Quote($var, true);
			$this->db->setQuery($query);
			$varObj = $this->db->loadObject();
			if (!empty($varObj))
			{
				return ($varObj->status == 3) ? true : false;
			}
		}
		$query = "SELECT `status` FROM `#__osefirewall_vars` WHERE `keyname` = " . $this->db->Quote($var, true);
		$this->db->setQuery($query);
		$result = $this->db->loadResult();
		return ($result == 3) ? true : false;
	}
	// Functions to scan layer 1 attacks //
	public function ScanLayer1() {
		$switch = '';
		$violation = '';
		$urlQuery = htmlspecialchars(str_replace(
						array ("%3C","%3E","%5C"," ","\\\\","%26","%3F"),
						array ("<",">","\\","%20","\\","&","?"),
					$_SERVER['QUERY_STRING']));
		$layer1Rules = $this->getSignatureKeys(1, true);
		// Loop through the exploit list and compare it to the query string
		foreach ($layer1Rules as $signature) {
			if (!empty ($urlQuery) && stristr($urlQuery, htmlspecialchars($signature->signature))) {
				// Check whitelist key patterns;
				$whitelisted = $this->checkIsWhitelisted($urlQuery);
				if ($whitelisted == false) {
					$this->ScanReturn['signature'] = $signature->signature;
					$this->ScanReturn['rule_id'] = $signature->id;
					$this->ScanReturn['detcontent_content'] = $urlQuery;
					$this->ScanReturn['impact'] = 100;
					break;
				}
			}
		}
		unset($layer1Rules);
		unset($urlQuery);
		unset($whitelisted);
		return $this->ScanReturn;
	}
	private function checkIsWhitelisted($urlQuery) {
		$wlkeys = $this->getSignatureKeys(0, false);
		if (empty ($result)) {
			return false;
		}
		$pattern = "/" . implode("|", $result) . "/";
		$matches = array ();
		preg_match($pattern, $urlQuery, $matches);
		if (count($matches) > 0) {
			return true;
		} else {
			return false;
		}
	}
	private function getSignatureKeys($action = 1, $obj = true) {
		$query = "SELECT `signature`,`id` FROM `#__osefirewall_signatures` WHERE `action` = " . (int) $action;
		$this->db->setQuery($query);
		if ($obj == true) {
			$results = $this->db->loadObjectList();
		} else {
			$results = $this->db->loadResultArray();
		}
		unset ($query);
		return $results;
	}
	private function cleanFileVariable($fileVar)
	{
		if (is_array($fileVar)) {
			foreach ($fileVar as $filetmp) {
							$fileVar= $filetmp;
			break;
			}
		}
		return $fileVar; 
	}
	// Function check uploaded files
	public function checkFileTypes($files) {
		if (!empty ($this->allowExts)) {
			foreach ($files as $file) {
				if (!empty ($file['tmp_name'])) {
					if (is_array($file['tmp_name']))
					{
						$file['tmp_name'] = $file['tmp_name'][0];
					}	
					if (!empty($file['tmp_name']))
					{	
						$file['tmp_name'] = $this->cleanFileVariable($file['tmp_name']);
						$file['type'] = $this->cleanFileVariable($file['type']);
						$mimeType = $this->getMimeType($file);
						$ext = explode('/', $file['type']);
						$allowExts = explode(",", trim($this->allowExts));
						$allowExts = array_map('trim', $allowExts);
						if ($ext[1] == 'vnd.openxmlformats-officedocument.wordprocessingml.document' && ($mimeType[1] != $ext[1])) {
							$ext[1] = 'msword';
						}
						if ($ext[1] != $mimeType[1]) {
							$this->ScanReturn['rule_id'] = 1;
							$this->ScanReturn['detcontent_content'] = $file['name'];
							$this->ScanReturn['impact'] = 100;
							if (is_array($_FILES['tmp_name']))
							{
									foreach ($_FILES['tmp_name'] as $filetmp)
									{	
										unlink($filetmp);
										break;
									}
							}
							else
							{
									unlink($_FILES['tmp_name']);
							}
							unset ($_FILES);
							return $this->ScanReturn; 
						}
						elseif (in_array($mimeType[1], $allowExts) == false) {
							$this -> show403('The upload of this file type ' . $mimeType[1] . ' is not allowed this website. If you are the server administrator, please add the extensions in the configuraiton in OSE Security first.');
						}
						else
						{
							return false; 
						}
					}
				}
			}
		}
	}
	private function getMimeType($file) {
		$mimeType = $this->getFileInfo($file['tmp_name']);
		if (empty ($mimeType)) {
			$mimeType = $this -> checkisPHPfile($file['tmp_name']);
		}
		if (!empty ($mimeType)) {
			if (strstr($mimeType, '/')!=false)
			{
				$mimeType = explode("/", $mimeType);
			}
			else
			{
				$tmp = explode(" ", $mimeType);
				$mimeType = array(); 
				$mimeType[0] = strtolower ($tmp [1]);
				$mimeType[1] = strtolower ($tmp [0]);
			}
		} else {
			$mimeType = explode("/", $file['type']);
		}
		return $mimeType;
	}
	private function getFileInfo($filename) {
		if (!defined('FILEINFO_MIME_TYPE')) {
			define('FILEINFO_MIME_TYPE', 1);
		}
		$defined_functions = get_defined_functions();
		if ((in_array('finfo_open', $defined_functions['internal'])) || function_exists('finfo_open')) {
			$finfo = finfo_open(FILEINFO_MIME_TYPE);
			$content_type = finfo_file($finfo, $filename);
			finfo_close($finfo);
			return $content_type;
		}
		elseif (function_exists('mime_content_type')) {
			$content_type = mime_content_type($filename);
			return $content_type;
		} else {
			return false;
		}
	}
	private function checkisPHPfile($file) {
		if (empty($file)) {
			return false; 
		}
		if (filesize($file) > '2048000') {
			return false;
		}
		$data = file($file);
		$data = implode("\r\n", $data);
		$pattern = "/(\<\?)|(\<\?php)/";
		if (preg_match($pattern, $data)) {
			return 'application/x-httpd-php';
		} else {
			return false;
		}
	}
	public function scanFileVirus($files) {
		if (!empty ($this->scanFileVirus)) {
			if (file_exists(OSECPU_PATH . OSEDS . "virusscan" . OSEDS . "virusscan.php")) 
			{
				require_once (OSECPU_PATH. OSEDS . "virusscan" . OSEDS . "virusscan.php");
				if (file_exists(OSECPU_PATH. OSEDS . "filescan" . OSEDS . "filescan.php")) 
				{
					require_once (OSECPU_PATH . OSEDS . "filescan" . OSEDS . "filescan.php");
				}
				$virScanClass = new oseVirusscan();
				$virScan = $virScanClass->getInstance();
				foreach ($files as $file) {
					if (!empty ($file['tmp_name'])) {
						$scanResult = $virScan->virusScan($file['tmp_name']);
						if ($scanResult == true) {
							$this->ScanReturn['rule_id'] = 1;
							$this->ScanReturn['detcontent_content'] = $file['name'];
							$this->ScanReturn['impact'] = 100;
							unlink($_FILES['tmp_name']);
							unset ($_FILES);
						}
					}
				}
			}
		}
	}
	
	public function scanFlooding() {
		require_once (OSEFIREWALL_PATH . OSEDS . 'includes'. OSEDS. 'scanner_addon.php');
		$result = OSEAH_Addons :: floodingCheck($this->decduration, $this->maxvisits, $this->ip, $this->db);
		if ($result == true) {
				$this->ScanReturn['rule_id'] = null;
				$this->ScanReturn['detcontent_content'] = null;
				$this->ScanReturn['impact'] = 10;
				return $this->ScanReturn;
		}
	}
	
	private function show403($msg) {
		header('HTTP/1.1 403 Forbidden');
		$banbody  = "<html>
						<head>
							<title>403 Forbidden</title>
						</head>
						<body>
								".$msg."
						</body>
					 </html>";
		echo $banbody;
		exit;
	}
}